#include <stdio.h>
#include <ctype.h>
#include <X/Xlib.h>
#include <sys/ioctl.h>

#define MAXWIN 24

struct window {
	Window win;
	WindowInfo winf;
	char *name;
	char *assoc_name;
} windows[MAXWIN], savedwins[MAXWIN];

int
get_windows()
{
	int n, i, j;
	Window *children, par;
	WindowInfo winf;

	XQueryTree(RootWindow, &par, &n, &children);
	for (i = 0, j = 0; i < n; i++)
	  {
	    XQueryWindow(children[i], &winf);
	    windows[j].winf = winf;
	    windows[j].win = children[i];
	    XFetchName(children[i],&windows[j].name);
	    if(winf.assoc_wind != (Window)NULL)
	      XFetchName(winf.assoc_wind,&windows[j].assoc_name);
	    savedwins[j] = windows[j];
	    j++;
	  }
	return(j);
}

main(argc,argv)
     int argc;
     char **argv;
{
  int numwins,i=0;
  struct sgttyb mode, omode;

  ioctl(1, TIOCGETP, &mode);
  omode=mode;
  mode.sg_flags |= CBREAK;
  ioctl(1, TIOCSETN, &mode);

  XOpenDisplay(argc > 1 ? argv[1] : (char *)NULL);
  numwins = get_windows();


  while(i>=0)
    {
      int stay=1, comm;

      disp_win(i);
      while(stay)
	{
	  printf("---->");
	  comm=getchar();
	  if (comm<32) comm+=64; /* map up control chars */
	  if (islower(comm)) comm = toupper(comm); /* map down uppercase */
	  switch(comm)
	    {
	    case 'U':
	      XUnmapWindow(windows[i].win);
	      break;
	    case 'M':
	      XMapWindow(windows[i].win);
	      break;
	    case 'D':
	      XDestroyWindow(windows[i].win);
	      stay = 0;
	      break;
	    case 'S':
	      XChangeWindow(windows[i].win,
			    getval("Width"),
			    getval("Height"));
	      break;
	    case 'V':
	      XMoveWindow(windows[i].win,
			  getval("x"),
			  getval("y"));
	      break;
	    case 'N': i++; stay=0; break;
	    case 'P': i--; stay=0; break;
	    case 'Q': i= -2; stay=0; break;
	    case 'L':
	      printf("\n");
	      disp_win(i);
	      break;
	    case 'R':
	      numwins = get_windows();
	      stay=0;
	      if(i>=numwins) i=0;
	      break;
	    default:
	      break;
	    }			/* switch */
	  XFlush();
	  printf("\n");
	}			/* while(stay) */
      if(i==-1) i=numwins-1;
      if(i>=numwins) i=0;
    }				/* while (i>=0) */
  ioctl(1, TIOCSETN, &omode);
}


int getval(query)
     char *query;
{
  char s[100],*sp;

  printf("%s?\n",query);
  sp=s;
  while(( (*sp++)=getchar() )!='\n');
  return(atoi(s));
}





disp_win(i)
     int i;
{
  struct window thiswin;

  thiswin = windows[i];
  /* display the window record */
  printf("[%d]window id 0x%x is called \"%s\"\n",
	 i,
	 thiswin.win,
	 thiswin.name);
  printf("Geometry =%dx%d+%d+%d, ",
	 thiswin.winf.width,
	 thiswin.winf.height,
	 thiswin.winf.x,
	 thiswin.winf.y);
  printf("Border %d pixels wide\n",
	 thiswin.winf.bdrwidth);
  printf("Window ");
  switch(thiswin.winf.mapped)
    {
    case IsUnmapped:
      printf("IsUnmapped"); break;
    case IsMapped:
      printf("IsMapped"); break;
    case IsInvisible:
      printf("IsInvisible"); break;
    default:
      printf("has bad Map status = 0x%x",
	     thiswin.winf.mapped);
    }
  printf(", Type ");
  switch(thiswin.winf.type)
    {
    case IsTransparent:
      printf("IsTransparent\n"); break;
    case IsOpaque:
      printf("IsOpaque\n"); break;
    case IsIcon:
      printf("IsIcon\n"); break;
    default:
      printf("has bad Map status = 0x%x\n",
	     thiswin.winf.type);
    }
  if(thiswin.winf.assoc_wind != (Window)NULL)
    printf("Associated window id 0x%x is called \"%s\"\n",
	   thiswin.winf.assoc_wind,
	   thiswin.assoc_name);
}
